#include <Rcpp.h>
#include <gperftools/profiler.h>
using namespace Rcpp;


// [[Rcpp::export]]
List sampleCpp2(List disjointList, NumericVector ages, 
			 NumericVector ageError, int nsim) {
  // disjointList is the list of elements
  // which cannot share a neighbour.
  // ages are the ages in order of disjointList
  // ageError are the errors (assumed to be sd) of
  // the measurement.
  int nobs = disjointList.size();
  NumericVector partitionVec(nobs);
  NumericMatrix partitionMat(nsim, nobs);
  NumericMatrix groupMeans(nsim, nobs);
  NumericVector accepted(nobs);
  double sigmak = 0.0;
  NumericVector bk(nsim);
  int sim;
  ProfilerStart("/home/robert/profout.log");
  for(sim=0; sim<nsim; sim++) {
    for(int j=0; j<nobs; j++) {
      partitionVec[j] = 0;
      accepted[j] = 0;
    }
    //partitionVec = 0;
    //accepted = 0;
    partitionVec[0] = 1;
    // now we sample from our candidates
    // which are all the unique values in partionvec,
    // plus 1 more than the maximum
    // I'd like all the values that are > 1, 
    // up to a maximum of nobs
    for(int i=1;i<nobs; i++) {
      NumericVector validobs = clone(partitionVec);
      int nextval = max(validobs) + 1;
      SEXP ll = disjointList[i];
      NumericVector y(ll);
      for(int j=0;j<nobs;j++) {
        for(int k=0;k<y.size();k++) {
  	if(partitionVec[j]==partitionVec[y[k]-1]) {
  	  //not allowed!
  	  validobs[j] = 0;
  	}
        }
      }
      NumericVector uniqueobs = sort_unique(validobs);
      int nvals = uniqueobs.size();
      NumericVector candidates(nvals);
      // and put the unique ones in
      // note we ignore the first observation
      // which will always? be zero
      for(int j=0;j<(nvals-1);j++) {
        candidates[j] = uniqueobs[j+1];
      }
      candidates[nvals-1] = nextval;
      //for(int k=0;k<nvals;k++) {
      //  Rcout << candidates[k] << " ";
      //}
      //Rcout << std::endl;
      // eg 1,2
      // now sample from candidates
      int randint;
      double randfloat;
      randfloat = runif(1)[0]*(nvals-1) + 0.5;
      randint = (int) randfloat;
      partitionVec[i] = candidates[randint];
    }
    // now calculate the pooled mean and sd.
    // we just allocate enough space for each
    // unique obs, (there will be max(partitionVec)
    // of these
    int ngroups = max(partitionVec);
    NumericVector grandMean(ngroups);
    NumericVector nInGroup(ngroups);
    NumericVector combinedVar(ngroups);
    sigmak = 0.0;
    for(int j=0;j<nobs;j++) {
       int vecIndex = partitionVec[j] - 1;
       grandMean[vecIndex] = grandMean[vecIndex] + ages[j];
       ++nInGroup[vecIndex];
       combinedVar[vecIndex] = combinedVar[vecIndex] + (ageError[j]*ageError[j] + ages[j]*ages[j]);
       // and put the selected partionVec into the matrix
       partitionMat(sim,j) =  partitionVec[j];
    }
     for(int j=0;j<ngroups;j++) {
       grandMean[j] = grandMean[j]/nInGroup[j];
       combinedVar[j] = (combinedVar[j]/nInGroup[j]) - (grandMean[j]*grandMean[j]);
       sigmak = sigmak + (combinedVar[j] * nInGroup[j]);
       // and store the group means
       groupMeans(sim,j) = grandMean[j];
     }
     bk[sim] =nobs*log(sigmak/nobs) + ngroups*log(nobs);
     //if (sim % 100 == 0) {
     //  Rcout << "sim " << sim <<std::endl;
     //}
  }
 ProfilerStop();
  return List::create(Rcpp::Named( "Bk") = bk,
		      Rcpp::Named( "partitions") = partitionMat,
		      Rcpp::Named( "groupMeans") = groupMeans);
}


